#ifndef _RATIONAL_HPP_
#define _RATIONAL_HPP_

#include <string>
#include <sstream>
#include <cmath>
#include <stdexcept>

/**
 * @author Marcell Dietl <liam [at] marcell-dietl.de [dot] de>
 *
 * Repraesentiert einen Bruch und stellt alle gaengigen Operatoren zur Verfuegung.
 */
class Rational
{
	private:
		int zaehler;
		int nenner;

		void pruefeNenner() const;
		int ggT(int a, int b) const;
		int kgV(int a, int b) const;
	public:
		/**
		 * Erzeugt einen Bruch mit Zaehler und Nenner.
		 *
		 * @param z Zaehler, der anfangs auf 0 gesetzt wird.
		 * @param n Nenner, der anfangs auf 1 gesetzt wird.
		 * @exception std::invalid_argument Exception, wenn im Nenner null steht.
		 */
		explicit Rational(int z=0, int n=1);

		/**
		 * Gibt den Wert des Zaehlers zurueck.
		 *
		 * @return Aktueller Wert des Zaehlers.
		 */
		int getZaehler();

		/**
		 * Gibt den Wert des Nenners zurueck.
		 *
		 * @return Aktueller Wert des Nenners.
		 */
		int getNenner();

		/**
		 * Setzt den Zaehler auf einen gewaehlten Wert.
		 *
		 * @param z Wert, auf den der Zaehler gesetzt wird.
		 */
		void setZaehler(int z);

		/**
		 * Setzt den Nenner auf einen gewaehlten Wert.
		 *
		 * @param n Wert, auf den der Nenner gesetzt wird.
		 * @exception std::invalid_argument Exception, wenn im Nenner null steht.
		 */
		void setNenner(int n);

		/**
		 * Wandelt den Bruch in eine Zeichenkette um.
		 *
		 * @return Zaehler und Nenner als std::string.
		 */
		std::string toString() const;

		/**
		 * Multipliziert zwei Brueche mittels ' * '.
		 *
		 * @param rechts Die rechte Seite der Multiplikation 'Bruch * Bruch'.
		 * @return Rational-Objekt des neu entstandenen Bruches.
		 */
		Rational operator*(const Rational& rechts) const;

		/**
		 * Multipliziert zwei Brueche mittels ' *= '.
		 *
		 * @param rechts Die rechte Seite der Multiplikation 'Bruch *= Bruch'.
		 * @return Referenz auf das existierende linke Rational-Objekt.
		 */
		Rational& operator*=(const Rational& rechts);

		/**
		 * Addiert zwei Brueche mittels ' + '.
		 *
		 * @param rechts Die rechte Seite der Addition 'Bruch + Bruch'.
		 * @return Rational-Objekt des neu entstandenen Bruches.
		 */
		Rational operator+(const Rational& rechts) const;

		/**
		 * Addiert zwei Brueche mittels ' += '.
		 *
		 * @param rechts Die rechte Seite der Addition 'Bruch += Bruch'.
		 * @return Referenz auf das existierende linke Rational-Objekt.
		 */
		Rational& operator+=(const Rational& rechts);

		/**
		 * Subtrahiert zwei Brueche mittels ' - '.
		 *
		 * @param rechts Die rechte Seite der Subtraktion 'Bruch - Bruch'.
		 * @return Rational-Objekt des neu entstandenen Bruches.
		 */
		Rational operator-(const Rational& rechts) const;

		/**
		 * Subtrahiert zwei Brueche mittels ' -= '.
		 *
		 * @param rechts Die rechte Seite der Subtraktion 'Bruch -= Bruch'.
		 * @return Referenz auf das existierende linke Rational-Objekt.
		 */
		Rational& operator-=(const Rational& rechts);

		/**
		 * Dividiert zwei Brueche mittels ' / '.
		 *
		 * @param rechts Die rechte Seite der Division 'Bruch / Bruch'.
		 * @return Rational-Objekt des neu entstandenen Bruches.
		 * @exception std::invalid_argument Exception, wenn im Nenner null steht.
		 */
		Rational operator/(const Rational& rechts) const;

		/**
		 * Dividiert zwei Brueche mittels ' /= '.
		 *
		 * @param rechts Die rechte Seite der Division 'Bruch /= Bruch'.
		 * @return Referenz auf das existierende linke Rational-Objekt.
		 * @exception std::invalid_argument Exception, wenn im Nenner null steht.
		 */
		Rational& operator/=(const Rational& rechts);

		/**
		 * Ist-Gleich-Vergleich zweier Brueche mittels ' == '.
		 *
		 * @param rechts Die rechte Seite des Vergleichs 'Bruch == Bruch'.
		 * @return true, wenn beide Brueche gleich sind, sonst false.
		 */
		bool operator==(const Rational& rechts) const;

		/**
		 * Ist-Ungleich-Vergleich zweier Brueche mittels ' != '.
		 *
		 * @param rechts Die rechte Seite des Vergleichs 'Bruch != Bruch'.
		 * @return true, wenn beide Brueche ungleich sind, sonst false.
		 */
		bool operator!=(const Rational& rechts) const;

		/**
		 * Ist-Kleiner-Vergleich zweier Brueche mittels ' < '.
		 *
		 * @param rechts Die rechte Seite des Vergleichs 'Bruch < Bruch'.
		 * @return true, wenn der linke Bruch kleiner ist, sonst false.
		 */
		bool operator<(const Rational& rechts) const;

		/**
		 * Ist-Kleiner-Gleich-Vergleich zweier Brueche mittels ' <= '.
		 *
		 * @param rechts Die rechte Seite des Vergleichs 'Bruch <= Bruch'.
		 * @return true, wenn der linke Bruch kleiner ist/beide Brueche gleich sind, sonst false.
		 */
		bool operator<=(const Rational& rechts) const;

		/**
		 * Ist-Groesser-Vergleich zweier Brueche mittels ' > '.
		 *
		 * @param rechts Die rechte Seite des Vergleichs 'Bruch > Bruch'.
		 * @return true, wenn der linke Bruch groesser ist, sonst false.
		 */
		bool operator>(const Rational& rechts) const;

		/**
		 * Ist-Groesser-Gleich-Vergleich zweier Brueche mittels ' >= '.
		 *
		 * @param rechts Die rechte Seite des Vergleichs 'Bruch >= Bruch'.
		 * @return true, wenn der linke Bruch groesser ist/beide Brueche gleich sind, sonst false.
		 */
		bool operator>=(const Rational& rechts) const;
};

std::ostream& operator<<(std::ostream& os, const Rational& r);

#endif
